//----------------------------------------------------------------------------------------------
// BoardBoard Library : Silicon Labs 2.2.0
// Board Select       : Seeed Studio XIAO MG24 (Sense)
// Plotocol stack     : None 
//----------------------------------------------------------------------------------------------
// 2025/01/05 @msfujino

// IMPORTANT NOTE:Cannot upload new sketches during DeepSleep
// When PC0(D0)(0)==LOW and reset, enable to upload. After uploading, be sure to set PC0==HIGH and Power on Reset. 
// \Arduino15\packages\SiliconLabs\hardware\silabs\2.2.0\libraries\ArduinoLowPower\src\ArduinoLowPower.cpp:L234
//      Overrides the WEAK escapeHatch() function in main.cpp to provide a way to prevent
//      the device from going to EM4 too quickly. If the device enters EM4 sleep the programmer
//      is not able to communicate with it - and without this mechanism it can easily be bricked.
//      This function will run before any user code and keep the device awake when the built-in button
//      (or the configured escape pin) is pressed during startup so that the programmer has a chance to
//      communicate with it.
//      void escape_hatch()
// \Arduino15\packages\SiliconLabs\hardware\silabs\2.2.0\variants\xiao_mg24\pins_arduino.h:L114
//      #define DEEP_SLEEP_ESCAPE_PIN   D0 // PC0

// Reference Manual, 24.3.12.3 Pin Function Tables, Table 24.3. GPIO Alternate Function Table
// WAKE_UP PC0(ESCAPE), PC5(SCL), PC7(RX), PA5(MOSI), back side : PB1, PB3, PD2, PD5 
// When PC0 is used for the WAKE_UP function, set it LOW for a short time to distinguish it from the ESCAPR function.

// If set timer and external pin at the same time, wake up immediately

#include <Arduino.h>
#include "ArduinoLowPower.h"

// pin definition
#define WAKE_UP  PC0 // NOTE:LOW for a short time

// on board Flush SPI_1 pins
#define CS1      PA6   // (21)
#define CLK1     PA0   // (17), D17   
#define MOSI1    PB0   // (15), D15
#define MISO1    PB1   // (16), D16

// on board peripherals pins
#define IMU_EN   PD5   // (19)
#define MIC_EN   PC8   // (22)
#define VBAT_EN  PD3   // (25)
#define RFSW_EN  PB5   // (27)

// Flash commands
#define READ_DATA     0x03
#define WRITE_ENABLE  0x06
#define PAGE_PROGRAM  0x02
#define SECTOR_ERASE  0x20

// Flash functions
void sendSPI(byte data) {
  for (int i = 0; i < 8; i++) {
    digitalWrite(MOSI1, data & 0x80);
    data <<= 1;
    digitalWrite(CLK1, HIGH);
    delayMicroseconds(1);
    digitalWrite(CLK1, LOW);
    delayMicroseconds(1);
  }
}

void writeEnable() {
  digitalWrite(CS1, LOW);
  sendSPI(WRITE_ENABLE);
  digitalWrite(CS1, HIGH);
}

// ******************************************************************************
void setup()
{
  Serial.begin(115200);
  while(!Serial);
  delay(2000);
  Serial.println("---- Deep sleep with external wakeup ----");

  // Set up the wake-up pin
  pinMode(WAKE_UP, INPUT_PULLUP);
  LowPower.attachInterruptWakeup(WAKE_UP, nullptr, FALLING);

  // builtin LED
  pinMode(LED_BUILTIN, OUTPUT);
  digitalWrite(LED_BUILTIN, HIGH);

  // on board flash
  pinMode(CS1, OUTPUT);
  pinMode(CLK1, OUTPUT);
  pinMode(MOSI1, OUTPUT);
  pinMode(MISO1, INPUT);
  digitalWrite(CS1, HIGH);   // CS1 HIGH

  // on board peripherals
  pinMode(IMU_EN, OUTPUT);
  pinMode(MIC_EN, OUTPUT);
  pinMode(VBAT_EN, OUTPUT);
  pinMode(RFSW_EN, OUTPUT);
  digitalWrite(IMU_EN, LOW);   // IMU Power OFF
  digitalWrite(MIC_EN, LOW);   // MIC Power OFF
  digitalWrite(VBAT_EN, LOW);  // VBAT Power OFF
  digitalWrite(RFSW_EN, LOW);  // RFSW Power OFF

  // Flash Deep Power Down 
  writeEnable();
  digitalWrite(CS1, LOW);
  sendSPI(0xB9);
  digitalWrite(CS1, HIGH);
}

void loop()
{
  digitalWrite(LED_BUILTIN, LOW);
  delay(1000);
  digitalWrite(LED_BUILTIN, HIGH);
//  delay(100);

  Serial.println("go to Deep Sleep and wait for External Wakeup\n");
  Serial.end();

  LowPower.deepSleep();
}

